"""A terminal-based Markdown document viewer, written in Textual."""

__author__ = "Textualize, Inc"
__copyright__ = "Copyright Textualize, Inc"
__credits__ = ["Dave Pearson"]
__maintainer__ = "Dave Pearson"
__email__ = "dave@textualize.io"
__version__ = "0.6.0"
__licence__ = "MIT"



"""Provides tools for saving and loading application data."""

from .bookmarks import Bookmark, load_bookmarks, save_bookmarks
from .config import Config, load_config, save_config
from .history import load_history, save_history

__all__ = [
    "Bookmark",
    "Config",
    "load_bookmarks",
    "load_config",
    "load_history",
    "save_bookmarks",
    "save_config",
    "save_history",
]



"""Code for getting files from a forge."""

from __future__ import annotations

from httpx import URL, AsyncClient, HTTPStatusError

from .advertising import USER_AGENT


async def build_raw_forge_url(
    url_format: str,
    owner: str,
    repository: str,
    branch: str | None = None,
    desired_file: str | None = None,
) -> URL | None:
    """Attempt to get raw forge URL for the given file.

    Args:
        owner: The owner of the repository to look in.
        repository: The repository to look in.
        branch: The optional branch to look in.
        desired_file: Optional name of the file to go looking for.

    Returns:
        The URL for the file, or `None` if none could be guessed.

    If the branch isn't supplied then `main` and `master` will be tested.

    If the target file isn't supplied it's assumed that `README.md` is the
    target.
    """
    desired_file = desired_file or "README.md"
    async with AsyncClient() as client:
        for test_branch in (branch,) if branch else ("main", "master"):
            url = url_format.format(
                owner=owner,
                repository=repository,
                branch=test_branch,
                file=desired_file,
            )
            response = await client.head(
                url,
                follow_redirects=True,
                headers={"user-agent": USER_AGENT},
            )
            try:
                response.raise_for_status()
                return URL(url)
            except HTTPStatusError:
                pass
    return None


async def build_raw_github_url(
    owner: str,
    repository: str,
    branch: str | None = None,
    desired_file: str | None = None,
) -> URL | None:
    """Attempt to get the GitHub raw URL for the given file.

    Args:
        owner: The owner of the repository to look in.
        repository: The repository to look in.
        branch: The optional branch to look in.
        desired_file: Optional name of the file to go looking for.

    Returns:
        The URL for the file, or `None` if none could be guessed.

    If the branch isn't supplied then `main` and `master` will be tested.

    If the target file isn't supplied it's assumed that `README.md` is the
    target.
    """
    return await build_raw_forge_url(
        "https://raw.githubusercontent.com/{owner}/{repository}/{branch}/{file}",
        owner,
        repository,
        branch,
        desired_file,
    )


async def build_raw_gitlab_url(
    owner: str,
    repository: str,
    branch: str | None = None,
    desired_file: str | None = None,
) -> URL | None:
    """Attempt to get the GitLab raw URL for the given file.

    Args:
        owner: The owner of the repository to look in.
        repository: The repository to look in.
        branch: The optional branch to look in.
        desired_file: Optional name of the file to go looking for.

    Returns:
        The URL for the file, or `None` if none could be guessed.

    If the branch isn't supplied then `main` and `master` will be tested.

    If the target file isn't supplied it's assumed that `README.md` is the
    target.
    """
    return await build_raw_forge_url(
        "https://gitlab.com/{owner}/{repository}/-/raw/{branch}/{file}",
        owner,
        repository,
        branch,
        desired_file,
    )


async def build_raw_bitbucket_url(
    owner: str,
    repository: str,
    branch: str | None = None,
    desired_file: str | None = None,
) -> URL | None:
    """Attempt to get the BitBucket raw URL for the given file.

    Args:
        owner: The owner of the repository to look in.
        repository: The repository to look in.
        branch: The optional branch to look in.
        desired_file: Optional name of the file to go looking for.

    Returns:
        The URL for the file, or `None` if none could be guessed.

    If the branch isn't supplied then `main` and `master` will be tested.

    If the target file isn't supplied it's assumed that `README.md` is the
    target.
    """
    return await build_raw_forge_url(
        "https://bitbucket.org/{owner}/{repository}/raw/{branch}/{file}",
        owner,
        repository,
        branch,
        desired_file,
    )


async def build_raw_codeberg_url(
    owner: str,
    repository: str,
    branch: str | None = None,
    desired_file: str | None = None,
) -> URL | None:
    """Attempt to get the Codeberg raw URL for the given file.

    Args:
        owner: The owner of the repository to look in.
        repository: The repository to look in.
        branch: The optional branch to look in.
        desired_file: Optional name of the file to go looking for.

    Returns:
        The URL for the file, or `None` if none could be guessed.

    If the branch isn't supplied then `main` and `master` will be tested.

    If the target file isn't supplied it's assumed that `README.md` is the
    target.
    """
    return await build_raw_forge_url(
        "https://codeberg.org/{owner}/{repository}/raw//branch/{branch}/{file}",
        owner,
        repository,
        branch,
        desired_file,
    )



"""Provides the panes that go into the main navigation area."""

from .bookmarks import Bookmarks
from .history import History
from .local_files import LocalFiles
from .table_of_contents import TableOfContents

__all__ = [
    "Bookmarks",
    "History",
    "LocalFiles",
    "TableOfContents",
]



"""General utility and support code."""

from .forge import (
    build_raw_bitbucket_url,
    build_raw_codeberg_url,
    build_raw_github_url,
    build_raw_gitlab_url,
)
from .type_tests import is_likely_url, maybe_markdown

__all__ = [
    "build_raw_bitbucket_url",
    "build_raw_codeberg_url",
    "build_raw_github_url",
    "build_raw_gitlab_url",
    "is_likely_url",
    "maybe_markdown",
]



"""Provides the history navigation pane."""

from __future__ import annotations

from functools import partial
from pathlib import Path

from httpx import URL
from rich.text import Text
from textual.app import ComposeResult
from textual.binding import Binding
from textual.message import Message
from textual.widgets import OptionList
from textual.widgets.option_list import Option

from ...dialogs import YesNoDialog
from .navigation_pane import NavigationPane


class Entry(Option):
    """An entry in the history."""

    def __init__(self, history_id: int, location: Path | URL) -> None:
        """Initialise the history entry item.

        Args:
            history_id: The ID of the item of history.
            location: The location being added to history.
        """
        super().__init__(self._as_prompt(location))
        self.history_id = history_id
        """The ID of the item of history."""
        self.location = location
        """The location for his entry in the history."""

    @staticmethod
    def _as_prompt(location: Path | URL) -> Text:
        """Depict the location as a decorated prompt.

        Args:
            location: The location to depict.

        Returns:
            A prompt with icon, etc.
        """
        if isinstance(location, Path):
            return Text.from_markup(
                f":page_facing_up: [bold]{location.name}[/]\n[dim]{location.parent}[/]",
                overflow="ellipsis",
            )
        return Text.from_markup(
            f":globe_with_meridians: [bold]{Path(location.path).name}[/]"
            f"\n[dim]{Path(location.path).parent}\n{location.host}[/]",
            overflow="ellipsis",
        )


class History(NavigationPane):
    """History navigation pane."""

    DEFAULT_CSS = """
    History {
        height: 100%;
    }

    History > OptionList {
        background: $panel;
    }
    """

    BINDINGS = [
        Binding("delete", "delete", "Delete the history item"),
        Binding("backspace", "clear", "Clean the history"),
    ]
    """The bindings for the history navigation pane."""

    def __init__(self) -> None:
        """Initialise the history navigation pane."""
        super().__init__("History")

    def compose(self) -> ComposeResult:
        """Compose the child widgets."""
        yield OptionList()

    def set_focus_within(self) -> None:
        """Focus the option list."""
        self.query_one(OptionList).focus(scroll_visible=False)

    def update_from(self, locations: list[Path | URL]) -> None:
        """Update the history from the given list of locations.

        Args:
            locations: A list of locations to update the history with.

        This call removes any existing history and sets it to the given
        value.
        """
        option_list = self.query_one(OptionList).clear_options()
        for history_id, location in reversed(list(enumerate(locations))):
            option_list.add_option(Entry(history_id, location))

    class Goto(Message):
        """Message that requests the viewer goes to a given location."""

        def __init__(self, location: Path | URL) -> None:
            """Initialise the history goto message.

            Args:
                location: The location to go to.
            """
            super().__init__()
            self.location = location
            """The location to go to."""

    def on_option_list_option_selected(self, event: OptionList.OptionSelected) -> None:
        """Handle an entry in the history being selected.

        Args:
            event: The event to handle.
        """
        event.stop()
        assert isinstance(event.option, Entry)
        self.post_message(self.Goto(event.option.location))

    class Delete(Message):
        """Message that requests the viewer to delete an item of history."""

        def __init__(self, history_id: int) -> None:
            """initialise the history delete message.

            args:
                history_id: The ID of the item of history to delete.
            """
            super().__init__()
            self.history_id = history_id
            """The ID of the item of history to delete."""

    def delete_history(self, history_id: int, delete_it: bool) -> None:
        """Delete a given history entry.

        Args:
            history_id: The ID of the item of history to delete.
            delete_it: Should it be deleted?
        """
        if delete_it:
            self.post_message(self.Delete(history_id))

    def action_delete(self) -> None:
        """Delete the highlighted item from history."""
        history = self.query_one(OptionList)
        if (item := history.highlighted) is not None:
            assert isinstance(entry := history.get_option_at_index(item), Entry)
            self.app.push_screen(
                YesNoDialog(
                    "Delete history entry?",
                    "Are you sure you want to delete the history entry?",
                ),
                partial(self.delete_history, entry.history_id),
            )

    class Clear(Message):
        """Message that requests that the history be cleared."""

    def clear_history(self, clear_it: bool) -> None:
        """Perform a history clear.

        Args:
            clear_it: Should it be cleared?
        """
        if clear_it:
            self.post_message(self.Clear())

    def action_clear(self) -> None:
        """Clear out the whole history."""
        self.app.push_screen(
            YesNoDialog(
                "Clear history?",
                "Are you sure you want to clear everything out of history?",
            ),
            self.clear_history,
        )



"""Provides a dialog for getting a yes/no response from the user."""

from __future__ import annotations

from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Center, Horizontal, Vertical
from textual.screen import ModalScreen
from textual.widgets import Button, Static


class YesNoDialog(ModalScreen[bool]):
    """A dialog for asking a user a yes/no question."""

    DEFAULT_CSS = """
    YesNoDialog {
        align: center middle;
    }

    YesNoDialog > Vertical {
        background: $panel;
        height: auto;
        width: auto;
        border: thick $primary;
    }

    YesNoDialog > Vertical > * {
        width: auto;
        height: auto;
    }

    YesNoDialog Static {
        width: auto;
    }

    YesNoDialog .spaced {
        padding: 1;
    }

    YesNoDialog #question {
        min-width: 100%;
        border-top: solid $primary;
        border-bottom: solid $primary;
    }

    YesNoDialog Button {
        margin-right: 1;
    }

    YesNoDialog #buttons {
        width: 100%;
        align-horizontal: right;
        padding-right: 1;
    }
    """
    """The default CSS for the yes/no dialog."""

    BINDINGS = [
        Binding("left,up", "focus_previous", "", show=False),
        Binding("right,down", "focus_next", "", show=False),
        Binding("escape", "app.pop_screen", "", show=False),
    ]
    """Bindings for the yes/no dialog."""

    def __init__(  # pylint:disable=too-many-arguments
        self,
        title: str,
        question: str,
        yes_label: str = "Yes",
        no_label: str = "No",
        yes_first: bool = True,
    ) -> None:
        """Initialise the yes/no dialog.

        Args:
            requester: The widget requesting the input.
            title: The title for the dialog.
            question: The question to ask.
            yes_label: The optional label for the yes button.
            no_label: The optional label for the no button.
            yes_first: Should the yes button come first?
            cargo: Any cargo value for the question.
            id: The ID for the dialog.
        """
        super().__init__()
        self._title = title
        """The title for the dialog."""
        self._question = question
        """The question to ask the user."""
        self._aye = yes_label
        """The label for the yes button."""
        self._naw = no_label
        """The label for the no button."""
        self._aye_first = yes_first
        """Should the positive button come first?"""

    def compose(self) -> ComposeResult:
        """Compose the content of the dialog."""
        with Vertical():
            with Center():
                yield Static(self._title, classes="spaced")
            yield Static(self._question, id="question", classes="spaced")
            with Horizontal(id="buttons"):
                aye = Button(self._aye, id="yes")
                naw = Button(self._naw, id="no")
                if self._aye_first:
                    aye.variant = "primary"
                    yield aye
                    yield naw
                else:
                    naw.variant = "primary"
                    yield naw
                    yield aye

    def on_mount(self) -> None:
        """Configure the dialog once the DOM is ready."""
        self.query(Button).first().focus()

    def on_button_pressed(self, event: Button.Pressed) -> None:
        """Handle a button being pressed on the dialog.

        Args:
            event: The event to handle.
        """
        self.dismiss(event.button.id == "yes")



"""Provides the viewer's omnibox widget."""

from __future__ import annotations

from pathlib import Path
from re import compile as compile_regexp
from typing import Type
from webbrowser import open as open_url

from httpx import URL
from textual.message import Message
from textual.reactive import var
from textual.widgets import Input

from ..utility import is_likely_url
from ..utility.advertising import DISCORD, ORGANISATION_NAME, PACKAGE_NAME


class Omnibox(Input):
    """The command and location input widget for the viewer."""

    DEFAULT_CSS = """
    Omnibox {
        dock: top;
        padding: 0;
        height: 3;
    }

    Omnibox .input--placeholder {
        color: $text 50%;
    }
    """
    """Default styling for the omnibox."""

    visiting: var[str] = var("")
    """The location that is being visited."""

    def watch_visiting(self) -> None:
        """Watch the visiting reactive variable."""
        self.placeholder = self.visiting or "Enter a location or command"
        if self.visiting:
            self.value = self.visiting

    _ALIASES: dict[str, str] = {
        "a": "about",
        "b": "bookmarks",
        "bm": "bookmarks",
        "bb": "bitbucket",
        "c": "contents",
        "cb": "codeberg",
        "cd": "chdir",
        "cl": "changelog",
        "gh": "github",
        "gl": "gitlab",
        "h": "history",
        "l": "local",
        "obs": "obsidian",
        "toc": "contents",
        "q": "quit",
        "?": "help",
    }
    """Command aliases."""

    @staticmethod
    def _split_command(value: str) -> list[str]:
        """Split a value into a command and argument tail.

        Args:
            value: The value to split.

        Returns:
            A list of the command and the argument(s).
        """
        command = value.split(None, 1)
        return [*command, ""] if len(command) == 1 else command

    def _is_command(self, value: str) -> bool:
        """Is the given string a known command?

        Args:
            value: The value to check.

        Returns:
            `True` if the string is a known command, `False` if not.
        """
        command, *_ = self._split_command(value)
        return (
            getattr(self, f"command_{self._ALIASES.get(command, command)}", None)
            is not None
        )

    def _execute_command(self, command: str) -> None:
        """Execute the given command.

        Args:
            command: The comment to execute.
        """
        command, arguments = self._split_command(command)
        getattr(self, f"command_{self._ALIASES.get(command, command)}")(
            arguments.strip()
        )

    class LocalViewCommand(Message):
        """The local file view command."""

        def __init__(self, path: Path) -> None:
            """Initialise the local view command.

            Args:
                path: The path to view.
            """
            super().__init__()
            self.path = path
            """The path of the file to view."""

    class RemoteViewCommand(Message):
        """The remote file view command."""

        def __init__(self, url: URL) -> None:
            """Initialise the remove view command.

            Args:
                url: The URL of the remote file to view.
            """
            super().__init__()
            self.url = url
            """The URL of the file to view."""

    class LocalChdirCommand(Message):
        """Command for changing the local files directory."""

        def __init__(self, target: Path) -> None:
            """Initialise the local files chdir command."""
            super().__init__()
            self.target = target
            """The target directory to change to."""

    def on_input_submitted(self, event: Input.Submitted) -> None:
        """Handle the user submitting the input.

        Args:
            event: The submit event.
        """

        # Clean up whatever the user input.
        submitted = self.value.strip()

        # Now that we've got it, empty the value. We'll put it back
        # depending on the outcome.
        self.value = ""

        # Work through the possible options for what the user entered.
        if is_likely_url(submitted):
            # It looks like it's an URL of some description so try and load
            # it as such.
            self.post_message(self.RemoteViewCommand(URL(submitted)))
        elif (path := Path(submitted).expanduser().resolve()).exists():
            # It's a match for something in the local filesystem. Is it...
            if path.is_file():
                # a file! Try and open it for viewing.
                self.post_message(self.LocalViewCommand(path))
                self.value = str(path)
            elif path.is_dir():
                # Nope, it's a directory. Take that to be a request to open
                # the local file selection navigation pane with the
                # directory as the root.
                self.post_message(self.LocalChdirCommand(path))
            else:
                # It's something that exists in the filesystem, but it's not
                # a directory or a file. Let's nope on that for now.
                return
        elif self._is_command(command := submitted.lower()):
            # Having checked for URLs and existing filesystem things, it's
            # now safe to look for commands. Having got here, it is a match
            # for a command so we handle it as such.
            self._execute_command(command)
        else:
            # Having got this far, the best thing to do now is assume that
            # the user was attempting to enter a filename to view and got it
            # wrong. So that they get some sort of feedback, let's attempt
            # to view it anyway.
            self.post_message(self.LocalViewCommand(Path(submitted)))
            # Because it'll raise an error and the user may want to edit the
            # input to get it right, we put the original input back in
            # place.
            self.value = submitted

        # If we got a match above stop the event.
        event.stop()

    class ContentsCommand(Message):
        """The table of contents command."""

    def command_contents(self, _: str) -> None:
        """Handle the table of contents command."""
        self.post_message(self.ContentsCommand())

    class LocalFilesCommand(Message):
        """The local files command."""

    def command_local(self, _: str) -> None:
        """View the local files."""
        self.post_message(self.LocalFilesCommand())

    class BookmarksCommand(Message):
        """The bookmarks command."""

    def command_bookmarks(self, _: str) -> None:
        """View the bookmarks."""
        self.post_message(self.BookmarksCommand())

    class QuitCommand(Message):
        """The quit command."""

    def command_quit(self, _: str) -> None:
        """The quit command."""
        self.post_message(self.QuitCommand())

    class HistoryCommand(Message):
        """The history command."""

    def command_history(self, _: str) -> None:
        """The history command."""
        self.post_message(self.HistoryCommand())

    class AboutCommand(Message):
        """The about command."""

    def command_about(self, _: str) -> None:
        """The about command."""
        self.post_message(self.AboutCommand())

    class HelpCommand(Message):
        """The help command."""

    def command_help(self, _: str) -> None:
        """The help command."""
        self.post_message(self.HelpCommand())

    def command_chdir(self, target: str) -> None:
        """The chdir command.

        Args:
            target: The target directory to change to.
        """
        self.post_message(
            self.LocalChdirCommand(Path(target or "~").expanduser().resolve())
        )

    _GUESS_BRANCH = compile_regexp(
        r"^(?P<owner>[^/ ]+)[/ ](?P<repo>[^ :]+)(?: +(?P<file>[^ ]+))?$"
    )
    """Regular expression for matching a repo and file where we'll guess the branch."""

    _SPECIFIC_BRANCH = compile_regexp(
        r"^(?P<owner>[^/ ]+)[/ ](?P<repo>[^ :]+):(?P<branch>[^ ]+)(?: +(?P<file>[^ ]+))?$"
    )
    """Regular expression for matching a repo and file where the branch is also given."""

    class ForgeCommand(Message):
        """The base git forge quick load command."""

        def __init__(
            self,
            owner: str,
            repository: str,
            branch: str | None = None,
            desired_file: str | None = None,
        ) -> None:
            """Initialise the git forge quick load command."""
            super().__init__()
            self.owner = owner
            """The owner of the repository."""
            self.repository = repository
            """The repository."""
            self.branch: str | None = branch
            """The optional branch to attempt to pull the file from."""
            self.desired_file: str | None = desired_file
            """The optional file the user wants from the repository."""

    def _forge_quick_look(self, command: Type[ForgeCommand], tail: str) -> None:
        """Core forge quick look support method.

        Args:
            command: The command message to be posted.
            tail: The tail of the command to be parsed.
        """
        tail = tail.strip()
        if hit := self._GUESS_BRANCH.match(tail):
            self.post_message(
                command(hit["owner"], hit["repo"], desired_file=hit["file"])
            )
        elif hit := self._SPECIFIC_BRANCH.match(tail):
            self.post_message(
                command(
                    hit["owner"],
                    hit["repo"],
                    branch=hit["branch"],
                    desired_file=hit["file"],
                )
            )

    class GitHubCommand(ForgeCommand):
        """The GitHub quick load command."""

    def command_github(self, tail: str) -> None:
        """The github command.

        Args:
            tail: The tail of the command.
        """
        self._forge_quick_look(self.GitHubCommand, tail)

    class GitLabCommand(ForgeCommand):
        """The GitLab quick load command."""

    def command_gitlab(self, tail: str) -> None:
        """The Gitlab command.

        Args:
            tail: The tail of the command.
        """
        self._forge_quick_look(self.GitLabCommand, tail)

    class BitBucketCommand(ForgeCommand):
        """The BitBucket quick load command."""

    def command_bitbucket(self, tail: str) -> None:
        """The BitBucket command.

        Args:
            tail: The tail of the command.
        """
        self._forge_quick_look(self.BitBucketCommand, tail)

    class CodebergCommand(ForgeCommand):
        """The Codeberg quick load command."""

    def command_codeberg(self, tail: str) -> None:
        """The Codeberg command.

        Args:
            tail: The tail of the command.
        """
        self._forge_quick_look(self.CodebergCommand, tail)

    def command_discord(self, _: str) -> None:
        """The command to visit the Textualize discord server."""
        open_url(DISCORD)

    def command_changelog(self, _: str) -> None:
        """The command to show the application's own ChangeLog"""
        self.command_github(f"{ORGANISATION_NAME}/{PACKAGE_NAME} ChangeLog.md")

    def command_obsidian(self, vault: str) -> None:
        """The command to visit an obsidian vault, if one can be seen.

        Args:
            vault: The vault to visit.

        If the vault name is empty, an attempt will be made to visit the
        root level of all Obsidian vaults.

        Note:
            At the moment this will only work with Obsidian on macOS where
            the vaults are being held in iCloud.
        """
        # Right now this will only work on macOS. I've not used Obsidian on
        # any other OS so I'm unsure where the vault will be stored. I'll
        # add to this once I've found out.
        if (
            target := (
                Path(
                    "~/Library/Mobile Documents/iCloud~md~obsidian/Documents"
                ).expanduser()
                / vault
            )
        ).exists():
            self.command_chdir(str(target))



"""Provides the 'branding' for the application."""

from typing_extensions import Final

from .. import __version__

ORGANISATION_NAME: Final[str] = "textualize"
"""The organisation name to use when creating namespaced resources."""

ORGANISATION_TITLE: Final[str] = "Textualize"
"""The organisation title."""

ORGANISATION_URL: Final[str] = "https://www.textualize.io/"
"""The organisation URL."""

PACKAGE_NAME: Final[str] = "frogmouth"
"""The name of the package."""

APPLICATION_TITLE: Final[str] = "Frogmouth"
"""The title of the application."""

USER_AGENT: Final[str] = f"{PACKAGE_NAME} v{__version__}"
"""The user agent to use when making web requests."""

DISCORD: Final[str] = "https://discord.gg/Enf6Z3qhVr"
"""The link to the Textualize Discord server."""

TEXTUAL_URL: Final[str] = "https://textual.textualize.io/"
"""The URL people should visit to find out more about Textual."""



"""Provides the navigation panel widget."""

from __future__ import annotations

from pathlib import Path

from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Vertical
from textual.message import Message
from textual.reactive import var
from textual.widgets import TabbedContent, Tabs
from typing_extensions import Self

from ..data import load_config, save_config
from .navigation_panes.bookmarks import Bookmarks
from .navigation_panes.history import History
from .navigation_panes.local_files import LocalFiles
from .navigation_panes.navigation_pane import NavigationPane
from .navigation_panes.table_of_contents import TableOfContents


class Navigation(Vertical, can_focus=False, can_focus_children=True):
    """A navigation panel widget."""

    DEFAULT_CSS = """
    Navigation {
        width: 44;
        background: $panel;
        display: block;
        dock: left;
    }

    Navigation.hidden {
        display: none;
    }

    TabbedContent {
        height: 100% !important;
    }

    ContentSwitcher {
        height: 1fr !important;
    }
    """

    BINDINGS = [
        Binding("comma,a,ctrl+left,shift+left,h", "previous_tab", "", show=False),
        Binding("full_stop,d,ctrl+right,shift+right,l", "next_tab", "", show=False),
        Binding("\\", "toggle_dock", "Dock left/right"),
    ]
    """Bindings local to the navigation pane."""

    popped_out: var[bool] = var(False)
    """Is the navigation popped out?"""

    docked_left: var[bool] = var(True)
    """Should navigation be docked to the left side of the screen?"""

    def compose(self) -> ComposeResult:
        """Compose the content of the navigation pane."""
        self.popped_out = False
        # pylint:disable=attribute-defined-outside-init
        self._contents = TableOfContents()
        self._local_files = LocalFiles()
        self._bookmarks = Bookmarks()
        self._history = History()
        with TabbedContent() as tabs:
            self._tabs = tabs
            yield self._contents
            yield self._local_files
            yield self._bookmarks
            yield self._history

    def on_mount(self) -> None:
        """Configure navigation once the DOM is set up."""
        self.docked_left = load_config().navigation_left

    class Hidden(Message):
        """Message sent when the navigation is hidden."""

    def watch_popped_out(self) -> None:
        """Watch for changes to the popped out state."""
        self.set_class(not self.popped_out, "hidden")
        if not self.popped_out:
            self.post_message(self.Hidden())

    def toggle(self) -> None:
        """Toggle the popped/unpopped state."""
        self.popped_out = not self.popped_out

    def watch_docked_left(self) -> None:
        """Watch for changes to the left-docking status."""
        self.styles.dock = "left" if self.docked_left else "right"

    @property
    def table_of_contents(self) -> TableOfContents:
        """The table of contents widget."""
        return self._contents

    @property
    def local_files(self) -> LocalFiles:
        """The local files widget."""
        return self._local_files

    @property
    def bookmarks(self) -> Bookmarks:
        """The bookmarks widget."""
        return self._bookmarks

    @property
    def history(self) -> History:
        """The history widget."""
        return self._history

    def jump_to_local_files(self, target: Path | None = None) -> Self:
        """Switch to and focus the local files pane.

        Returns:
            Self.
        """
        if (
            self.popped_out
            and target is None
            and self.query_one(Tabs).active == self._local_files.id
        ):
            self.popped_out = False
        else:
            self.popped_out = True
            if target is not None:
                self._local_files.chdir(target)
            self._local_files.activate().set_focus_within()
        return self

    def jump_to_bookmarks(self) -> Self:
        """Switch to and focus the bookmarks pane.

        Returns:
            Self.
        """
        if self.popped_out and self.query_one(Tabs).active == self._bookmarks.id:
            self.popped_out = False
        else:
            self.popped_out = True
            self._bookmarks.activate().set_focus_within()
        return self

    def jump_to_history(self) -> Self:
        """Switch to and focus the history pane.

        Returns:
            Self.
        """
        if self.popped_out and self.query_one(Tabs).active == self._history.id:
            self.popped_out = False
        else:
            self.popped_out = True
            self._history.activate().set_focus_within()
        return self

    def jump_to_contents(self) -> Self:
        """Switch to and focus the table of contents pane.

        Returns:
            Self.
        """
        if self.popped_out and self.query_one(Tabs).active == self._contents.id:
            self.popped_out = False
        else:
            self.popped_out = True
            self._contents.activate().set_focus_within()
        return self

    def action_previous_tab(self) -> None:
        """Switch to the previous tab in the navigation pane."""
        self.query_one(Tabs).action_previous_tab()
        self.focus_tab()

    def action_next_tab(self) -> None:
        """Switch to the next tab in the navigation pane."""
        self.query_one(Tabs).action_next_tab()
        self.focus_tab()

    def action_toggle_dock(self) -> None:
        """Toggle the dock side for the navigation."""
        config = load_config()
        config.navigation_left = not config.navigation_left
        save_config(config)
        self.docked_left = config.navigation_left

    def focus_tab(self) -> None:
        """Focus the currently active tab."""
        if active := self.query_one(Tabs).active:
            self.query_one(
                f"NavigationPane#{active}", NavigationPane
            ).set_focus_within()



"""Provides useful dialogs for the application."""

from .error import ErrorDialog
from .help_dialog import HelpDialog
from .information import InformationDialog
from .input_dialog import InputDialog
from .yes_no_dialog import YesNoDialog

__all__ = [
    "ErrorDialog",
    "InformationDialog",
    "InputDialog",
    "HelpDialog",
    "YesNoDialog",
]
